@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2010-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2010-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -27,11 +27,13 @@ from hashlib import md5
 if os.name == 'nt':
     import winsound  # pylint: disable=import-error
 
-from common import Debug, Internal, nativeString
+from common import Debug, Internal, StrMixin
 from util import which, removeIfExists, uniqueList, elapsedSince
-from log import logWarning, m18n, logDebug, logException
+from log import logWarning, i18n, logDebug, logException
 
-from kde import KGlobal, cacheDir
+from qt import QStandardPaths
+
+from kde import cacheDir
 
 from tile import Tile
 
@@ -46,44 +48,38 @@ from tile import Tile
         # self.audio.play()
 
 
-class Sound(object):
+class Sound:
 
     """the sound interface. Use class variables and class methods,
     thusly ensuring no two instances try to speak"""
-    __oggName = None
+    __oggBinary = None
+    __bonusOgg = None
     playProcesses = []
     lastCleaned = None
 
     @staticmethod
-    def findOgg():
-        """sets __oggName to exe name or an empty string"""
-        if Sound.__oggName is None:
+    def findOggBinary():
+        """sets __oggBinary to exe name or an empty string"""
+        if Sound.__oggBinary is None:
             if os.name == 'nt':
-                parentDirectories = KGlobal.dirs().findDirs(
-                    "appdata", "voices")
-                if parentDirectories:
-                    oggName = os.path.join(parentDirectories[0], 'oggdec.exe')
-                    msg = ''  # we bundle oggdec.exe with the kajongg installer, it must be there
-                else:
-                    msg = m18n(
-                        'No voices will be heard because the program %1 is missing',
-                        'oggdec.exe')
+                Sound.__oggBinary = os.path.join('share', 'kajongg', 'voices', 'oggdec.exe')
+                msg = ''  # we bundle oggdec.exe with the kajongg installer, it must be there
             else:
-                oggName = 'ogg123'
-                msg = m18n(
+                oggBinary = 'ogg123'
+                msg = i18n(
                     'No voices will be heard because the program %1 is missing',
-                    oggName)
-            if which(oggName):
-                Sound.__oggName = oggName
-            else:
-                Sound.__oggName = ''
-                Internal.Preferences.useSounds = False
-                # checks again at next reenable
-                if msg:
-                    logWarning(msg)
+                    oggBinary)
+                if which(oggBinary):
+                    Sound.__oggBinary = oggBinary
+                else:
+                    Sound.__oggBinary = ''
+                    Internal.Preferences.useSounds = False
+                    # checks again at next reenable
+                    if msg:
+                        logWarning(msg)
             if Debug.sound:
-                logDebug('ogg123 found:' + Sound.__oggName)
-        return Sound.__oggName
+                logDebug('ogg123 found:' + Sound.__oggBinary)
+        return Sound.__oggBinary
 
     @staticmethod
     def cleanProcesses():
@@ -117,6 +113,7 @@ class Sound(object):
     @staticmethod
     def speak(what):
         """this is what the user of this module will call."""
+        # pylint: disable=too-many-branches
         if not Internal.Preferences.useSounds:
             return
         game = Internal.scene.game
@@ -129,22 +126,27 @@ class Sound(object):
                 reactor.callLater(1, Sound.speak, what)
                 return
         if os.path.exists(what):
-            oggName = Sound.findOgg()
-            if oggName:
+            oggBinary = Sound.findOggBinary()
+            if oggBinary:
                 if os.name == 'nt':
                     # convert to .wav, store .wav in cacheDir
                     name, ext = os.path.splitext(what)
-                    assert ext == '.ogg'
-                    nameParts = os.path.normpath(name).split(os.sep)
-                    nameParts = nameParts[nameParts.index('voices') + 1:]
+                    assert ext == '.ogg', 'what: {} name: {} ext: {}'.format(what, name, ext)
+                    if 'bell' in name:
+                        nameParts = ['bell']
+                    else:
+                        nameParts = os.path.normpath(name).split(os.sep)
+                        nameParts = nameParts[nameParts.index('voices') + 1:]
                     wavName = os.path.normpath(
                         '{}/{}.wav'.format(cacheDir(),
                                            '_'.join(nameParts)))
                     if not os.path.exists(wavName):
-                        args = [oggName, '-a', '-w', wavName, what]
+                        args = [oggBinary, '-a', '-w', wavName, os.path.normpath(what)]
                         startupinfo = subprocess.STARTUPINFO()
                         startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
                         subprocess.call(args, startupinfo=startupinfo)
+                        if Debug.sound:
+                            logDebug('converted {} to wav {}'.format(what, wavName))
                     try:
                         winsound.PlaySound(
                             wavName,
@@ -152,7 +154,7 @@ class Sound(object):
                     except RuntimeError:
                         pass
                 else:
-                    args = [oggName, '-q', what]
+                    args = [oggBinary, '-q', what]
                     if Debug.sound:
                         game.debug(' '.join(args))
                     process = subprocess.Popen(args)
@@ -161,27 +163,26 @@ class Sound(object):
                     Sound.playProcesses.append(process)
                     reactor.callLater(3, Sound.cleanProcesses)
                     reactor.callLater(6, Sound.cleanProcesses)
-#        else:
-#            text = os.path.basename(what)
-#            text = os.path.splitext(text)[0]
-            # If this ever works, we need to translate all texts
-            # we need package jovie and mbrola voices
-            # KSpeech setLanguage de
-            # KSpeech.showManagerDialog lets me define voices but
-            # how do I use them? it is always the same voice,
-            # setDefaultTalker "namefrommanager" does not change anything
-            # although defaultTalker returns what we just set even if no talker
-            # with that name exists
-            # getTalkerCodes returns nothing
-            # this all feels immature
-#            if len(text) == 2 and text[0] in 'sdbcw':
-#                text = Tile(text).name()
-#            args = ['qdbus', 'org.kde.jovie',
-#                    '/KSpeech', 'say', text, '1']
-#            subprocess.Popen(args)
 
+    @staticmethod
+    def bonus():
+        """ring some sort of bell, if we find such a file"""
+        if Sound.__bonusOgg is None:
+            Sound.__bonusOgg = ''
+            for oggName in (
+                    '/usr/share/sounds/KDE-Im-Message-In.ogg',
+                    'share/sounds/bell.ogg'):
+                if os.path.exists(oggName):
+                    Sound.__bonusOgg = oggName
+                    if Debug.sound:
+                        logDebug('Bonus sound found:{}'.format(oggName))
+                    break
+            if Debug.sound and not Sound.__bonusOgg:
+                logDebug('No bonus sound found')
+        if Sound.__bonusOgg:
+            Sound.speak(Sound.__bonusOgg)
 
-class Voice(object):
+class Voice(StrMixin):
 
     """this administers voice sounds.
 
@@ -191,13 +192,14 @@ class Voice(object):
     only sense to cache the voice in a tarfile at source."""
 
     __availableVoices = None
+    md5sumLength = 32 # magical constant
 
     def __init__(self, directory, content=None):
         """give this name a voice"""
         self.__md5sum = None
         if not os.path.split(directory)[0]:
             if Debug.sound:
-                logDebug(u'place voice %s in %s' % (directory, cacheDir()))
+                logDebug('place voice %s in %s' % (directory, cacheDir()))
             directory = os.path.join(cacheDir(), directory)
         self.directory = directory
         self.__setArchiveContent(content)
@@ -205,15 +207,12 @@ class Voice(object):
     def __str__(self):
         return self.directory
 
-    def __repr__(self):
-        return "<Voice: %s>" % self
-
     def language(self):
         """the language code of this voice. Locally defined voices
         have no language code and return 'local'.
         remote voices received from other clients return 'remote',
         they always get predecence."""
-        if len(self.directory) == 32:
+        if len(self.directory) == self.md5sumLength:
             if os.path.split(self.directory)[1] == self.md5sum:
                 return 'remote'
         if 'HOME' in os.environ:
@@ -236,12 +235,14 @@ class Voice(object):
         """a list of all voice directories"""
         if not Voice.__availableVoices:
             result = []
-            for parentDirectory in KGlobal.dirs().findDirs("appdata", "voices"):
-                parentDirectory = nativeString(parentDirectory)
+            directories = QStandardPaths.locateAll(
+                QStandardPaths.AppDataLocation, 'voices', QStandardPaths.LocateDirectory)
+            directories.insert(0, os.path.join('share', 'kajongg', 'voices'))
+            for parentDirectory in directories:
                 for (dirpath, _, _) in os.walk(parentDirectory, followlinks=True):
                     if os.path.exists(os.path.join(dirpath, 's1.ogg')):
                         result.append(Voice(dirpath))
-            group = KGlobal.config().group('Locale')
+            group = Internal.kajonggrc.group('Locale')
             prefLanguages = uniqueList(
                 ':'.join(['local', str(group.readEntry('Language')), 'en_US']).split(':'))
             prefLanguages = dict((x[1], x[0])
@@ -249,7 +250,7 @@ class Voice(object):
             result = sorted(
                 result, key=lambda x: prefLanguages.get(x.language(), 9999))
             if Debug.sound:
-                logDebug(u'found voices:%s' % [str(x) for x in result])
+                logDebug('found voices:%s' % [str(x) for x in result])
             Voice.__availableVoices = result
         return Voice.__availableVoices
 
@@ -262,17 +263,17 @@ class Voice(object):
             if name == voice.md5sum:
                 if Debug.sound:
                     logDebug(
-                        u'locate found %s by md5sum in %s' %
+                        'locate found %s by md5sum in %s' %
                         (name, voice.directory))
                 return voice
             elif name == dirname and voice.language() == 'local':
                 if Debug.sound:
                     logDebug(
-                        u'locate found %s by name in %s' %
+                        'locate found %s by name in %s' %
                         (name, voice.directory))
                 return voice
         if Debug.sound:
-            logDebug(u'%s not found' % (name))
+            logDebug('Personal sound for %s not found' % (name))
 
     def buildSubvoice(self, oggName, side):
         """side is 'left' or 'right'."""
@@ -298,11 +299,11 @@ class Voice(object):
             if callResult:
                 if Debug.sound:
                     logDebug(
-                        u'failed to build subvoice %s: return code=%s' %
+                        'failed to build subvoice %s: return code=%s' %
                         (angleName, callResult))
                 return stdName
             if Debug.sound:
-                logDebug(u'built subvoice %s' % angleName)
+                logDebug('built subvoice %s' % angleName)
         return angleName
 
     def localTextName(self, text, angle):
@@ -321,7 +322,7 @@ class Voice(object):
         fileName = self.localTextName(text, angle)
         if not os.path.exists(fileName):
             if Debug.sound:
-                logDebug(u'Voice.speak: fileName %s not found' % fileName)
+                logDebug('Voice.speak: fileName %s not found' % fileName)
         Sound.speak(fileName)
 
     def oggFiles(self):
@@ -343,7 +344,7 @@ class Voice(object):
             removeIfExists(self.archiveName())
             removeIfExists(md5FileName)
             self.__md5sum = None
-            logDebug(u'no ogg files in %s' % self)
+            logDebug('no ogg files in %s' % self)
             return
         md5sum = md5()
         for oggFile in ogg:
@@ -356,20 +357,20 @@ class Voice(object):
         if self.__md5sum != existingMd5sum:
             if Debug.sound:
                 if not os.path.exists(md5Name):
-                    logDebug(u'creating new %s' % md5Name)
+                    logDebug('creating new %s' % md5Name)
                 else:
                     logDebug(
-                        u'md5sum %s changed, rewriting %s with %s' %
+                        'md5sum %s changed, rewriting %s with %s' %
                         (existingMd5sum, md5Name, self.__md5sum))
             try:
-                open(md5Name, 'wb').write('%s\n' % self.__md5sum)
+                open(md5Name, 'w').write('%s\n' % self.__md5sum)
             except BaseException as exception:
                 logException(
-                    '\n'.join([m18n('cannot write <filename>%1</filename>: %2',
+                    '\n'.join([i18n('cannot write <filename>%1</filename>: %2',
                                     md5Name,
                                     str(exception)),
-                               m18n('The voice files have changed, their checksum has changed.'),
-                               m18n('Please reinstall kajongg or do, with sufficient permissions:'),
+                               i18n('The voice files have changed, their checksum has changed.'),
+                               i18n('Please reinstall kajongg or do, with sufficient permissions:'),
                                'cd {} ; cat *.ogg | md5sum > md5sum'.format(self.directory)]))
         if archiveExists:
             archiveIsOlder = os.path.getmtime(
@@ -401,7 +402,14 @@ class Voice(object):
     def savedmd5Sum(self):
         """returns the current value of the md5sum file"""
         if os.path.exists(self.md5FileName()):
-            return open(self.md5FileName(), 'r').readlines()[0].replace(' -', '').strip()
+            try:
+                line = open(self.md5FileName(), 'r').readlines()[0].replace(' -', '').strip()
+                if len(line) == self.md5sumLength:
+                    return line
+                else:
+                    logWarning('{} has wrong content: {}'.format(self.md5FileName(), line))
+            except BaseException as exc:
+                logWarning('{} has wrong content: {}'.format(self.md5FileName(), exc))
 
     @property
     def md5sum(self):
@@ -419,7 +427,7 @@ class Voice(object):
         tarFile = tarfile.open(mode='r|bz2', fileobj=filelike)
         tarFile.extractall(path=self.directory)
         if Debug.sound:
-            logDebug(u'extracted archive into %s' % self.directory)
+            logDebug('extracted archive into %s' % self.directory)
         tarFile.close()
         filelike.close()
 
